#pragma once

#include "Utils.h"
#include "VMMAPI.h"
#include "rmm.h"

/* Struct to hold Hyb, VMM and epics parameter values */
struct ParamValues {
  int param = -1;
  std::string value;
};

/** Class definition for the VMMTbl class */
class VMMTbl : public asynPortDriver {
 public:
  VMMTbl(RMM *rmm, const char *FENPortName, int ring, int node, int hybrids);

  /* Epics parameter management */
  asynStatus createEpicsParams();
  virtual asynStatus writeInt32(asynUser *pasynUser, epicsInt32 value);
  // virtual asynStatus writeFloat64(asynUser *pasynUser, epicsFloat64 value);
  virtual asynStatus writeInt8Array(asynUser *pasynUser, epicsInt8 *value, size_t nElements);
  virtual asynStatus readInt8Array(asynUser *pasynUser, epicsInt8 *value, size_t nElements, size_t *nIn);
  virtual asynStatus readInt32(asynUser *pasynUser, epicsInt32 *value);
  virtual asynStatus readFloat64(asynUser *pasynUser, epicsFloat64 *value);
  virtual asynStatus readOctet(asynUser *pasynUser, char *value, size_t maxChars, size_t *nActual, int *eomReason);
  asynStatus createParamAndStoreInVector(std::string paramName, asynParamType typ, std::vector<int> *vectorToStore);

 protected:
  int vmmRegBankVersion;
#define FIRST_VMM_PARAM vmmRegBankVersion
  int vmmSelectMonitorVMM0_;
  int vmmSelectMonitorVMM1_;
  int vmmProcessReadMonitorVMMs_;
  int vmmMonitorValueVMM0_;
  int vmmMonitorValueVMM1_;
  int vmmAcquire_;
  int vmmIsAcquiring_;
  int vmmNumHybrids;
  int IOCMessage;
  int global_SD;
  int global_SD0;
  int global_SD1;
  std::vector<int> vmmHybFwVersion_;
  std::vector<int> vmmHybId_;
  std::vector<int> vmmHybGeoPos_;
  std::vector<int> vmmHybLinkStatus_;
  std::vector<int> vmmHybSkew_;
  std::vector<int> vmmHybWidth_;
  std::vector<int> vmmHybPolarity_;
  std::vector<int> vmmHybEnable_;
  std::vector<std::vector<int>> vmmSC_, vmmSC_R;
  std::vector<std::vector<int>> vmmSL_, vmmSL_R;
  std::vector<std::vector<int>> vmmST_, vmmST_R;
  std::vector<std::vector<int>> vmmSTH_, vmmSTH_R;
  std::vector<std::vector<int>> vmmSM_, vmmSM_R;
  std::vector<std::vector<int>> vmmSD_, vmmSD_R;
  std::vector<std::vector<int>> vmmSMX_, vmmSMX_R;
  std::vector<std::vector<int>> vmmADCIDX_;
  std::vector<std::vector<int>> vmmADCVAL_;
  std::vector<std::vector<int>> vmmTemp_;
  std::vector<std::vector<int>> vmmTempTmstp_;
  std::vector<std::vector<int>> vmmBdg_;
  std::vector<std::vector<int>> vmmBdgTmstp_;
  std::vector<std::vector<int>> vmmThr_;
  std::vector<std::vector<int>> vmmThrTmstp_;
  std::vector<std::vector<int>> vmmPls_;
  std::vector<std::vector<int>> vmmPlsTmstp_;

 private:
  static constexpr const char *driverName = "VMMTbl";
  std::shared_ptr<VMMAPI> pVmmAPI;
};
